Package: svUnit
Type: Package
Version: 1.0.8
Title: 'SciViews::R' - Unit, Integration and System Testing
Description: A complete unit test system.
Authors@R: c(
  person("Philippe", "Grosjean", role = c("aut", "cre"),
    email = "phgrosjean@sciviews.org",
    comment = c(ORCID = "0000-0002-2694-9471")))
Maintainer: Philippe Grosjean <phgrosjean@sciviews.org>
Depends: R (>= 1.9.0)
Imports: utils (>= 1.9.0)
Suggests: svGUI (>= 1.0.0), datasets (>= 1.9.0), XML (>= 3.99.0.10),
        RUnit (>= 0.4.30), covr (>= 3.5.0), knitr (>= 1.42), rmarkdown
        (>= 2.21), spelling (>= 2.2.1)
License: GPL-2
URL: https://github.com/SciViews/svUnit,
        https://www.sciviews.org/svUnit/,
        https://sciviews.r-universe.dev/svUnit
BugReports: https://github.com/SciViews/svUnit/issues
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-US
ByteCompile: yes
NeedsCompilation: no
Packaged: 2025-08-26 10:27:46 UTC; phgrosjean
Author: Philippe Grosjean [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2694-9471>)
Repository: CRAN
Date/Publication: 2025-08-26 11:00:02 UTC
Built: R 4.6.0; ; 2026-04-25 16:34:16 UTC; unix
