# Generated by roxygen2: do not edit by hand

S3method(makeUnit,default)
S3method(makeUnit,svSuite)
S3method(makeUnit,svTest)
S3method(metadata,svSuiteData)
S3method(print,svSuite)
S3method(print,svSuiteData)
S3method(print,svTest)
S3method(print,svTestData)
S3method(protocol,default)
S3method(protocol,svSuiteData)
S3method(protocol_junit,svSuiteData)
S3method(protocol_junit,svTestData)
S3method(protocol_text,svSuiteData)
S3method(runTest,default)
S3method(runTest,list)
S3method(runTest,svSuite)
S3method(runTest,svTest)
S3method(stats,svSuiteData)
S3method(stats,svTestData)
S3method(summary,svSuiteData)
S3method(summary,svTestData)
export("test<-")
export(DEACTIVATED)
export(Log)
export(as.svSuite)
export(as.svTest)
export(checkEquals)
export(checkEqualsNumeric)
export(checkException)
export(checkIdentical)
export(checkTrue)
export(clearLog)
export(createLog)
export(errorLog)
export(guiSuiteAutoList)
export(guiSuiteList)
export(guiTestFeedback)
export(guiTestReport)
export(is.svSuite)
export(is.svSuiteData)
export(is.svTest)
export(is.svTestData)
export(is.test)
export(koUnit_isAutoTest)
export(koUnit_runTest)
export(koUnit_setAutoTest)
export(koUnit_showRUnitPane)
export(koUnit_version)
export(lastSuite)
export(lastTest)
export(makeTestListFromExamples)
export(makeUnit)
export(metadata)
export(protocol)
export(protocol_junit)
export(protocol_text)
export(runTest)
export(stats)
export(svSuite)
export(svSuiteList)
export(svTest)
export(test)
importFrom(utils,sessionInfo)
importFrom(utils,str)
